/* 
 * freememapplet - tray app to display free memory
 *
 * Copyright (C) 2006, Lior Tubi <lior2b@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
//MU
#include <eggtrayicon.h>
#include <eggtrayicon.c>
/****************************************************************************
 ****************************************************************************/

#define APPNAME    "freememapplet"
#define APPVERSION "1.2"
#define INTERVAL 30 /* (unsigned int) seconds. default update interval */
#define LOW_MEM 20 /* (unsigned int) MB */
#define LOW_MEM_FLASH 3 /* (unsigned int) MB. critical, flash display */
#define FLASH_INTERVAL 1500 /* (unsigned int) milliseconds */
#define FONT "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"

#define COLOR_NORMAL "#0E58B3"
#define COLOR_LOW "#7E026B"
#define COLOR_LOW_FLASH1 "#880000"
#define COLOR_LOW_FLASH2 "#D91418"

/****************************************************************************
 ****************************************************************************/

typedef enum {
	NORMAL = 0,
	LOW = 1,
	LOW_FLASH = 2,
} MemStatus;

/****************************************************************************
 ****************************************************************************/

void AppQuit(GtkWidget *widget, gpointer gdata);
gboolean Update(gpointer ptr);
void MemNormal(void);
void MemLow(void);
void MemLowFlash(void);
gboolean Flash(gpointer ptr);
gboolean IsCurrentBackground(GdkColor *color);
void SetBackground(GdkColor *color);
void ShowMenu(GtkWidget *widget, GdkEventButton *event, gpointer dummy);
void RunCommand(GtkWidget *widget, char *command);
void ExposeUpdate(GtkWidget *widget, GdkEvent *event, gpointer dummy);
void FormatKBytes(unsigned long long kbytes, char *buffer, int size);
char *strrepl(char *Str, size_t BufSiz, char *OldStr, char *NewStr);
/****************************************************************************
 ****************************************************************************/
 
GtkWidget *eventbox, *label;
GtkStyle *style;
GtkTooltips *tips;
GdkColor colorNormal, colorLow, colorLowFlash1, colorLowFlash2;
MemStatus status;
unsigned int flashTimeout;

 /****************************************************************************
 ****************************************************************************/
 
int main(int argc, char **argv) {

//MU
 static EggTrayIcon *window = NULL;

//	GtkWidget *window;

	GtkWidget  *hbox;
//MU	GdkFont *font;
	int i;
	unsigned int interval = INTERVAL;

	gtk_init(&argc, &argv);

	for(i = 1; i < argc; ++i) {
		if((!strcmp(argv[i], "-u") || !strcmp(argv[i], "--update")) && (i < argc-1)) {
			interval = (unsigned int)strtoul(argv[++i], NULL, 10);
		} else {
			if(!(!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version") || !strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))) {
				printf("%s: bad option or missing argument `%s`\n\n", argv[0], argv[i]);
			}

			printf("%s v%s by Lior Tubi <lior2b@gmail.com>\n" \
			"options:\n" \
			"  -u <seconds>, --upadte <seconds>: Set update interval (default %d)\n" \
			"  -h, --help, -v, --version:        Display this help text and exit\n" \
			, APPNAME, APPVERSION, INTERVAL);
			return 0;
		}
	}

//MU
//	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	window = egg_tray_icon_new("freemem");

	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(AppQuit), NULL);
//MU	gtk_window_set_title(GTK_WINDOW(window), APPNAME);
//MU	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);

	tips = gtk_tooltips_new();

	gdk_color_parse(COLOR_NORMAL, &colorNormal);
	gdk_color_parse(COLOR_LOW, &colorLow);
	gdk_color_parse(COLOR_LOW_FLASH1, &colorLowFlash1);
	gdk_color_parse(COLOR_LOW_FLASH2, &colorLowFlash2);

	label = gtk_label_new("");
	style = gtk_style_copy(gtk_widget_get_style(label));
	style->fg[GTK_STATE_NORMAL] = style->white;
//MU
/*
	if((font = gdk_font_load(FONT))) {
		style->font = font;
	}
*/ 


	gtk_widget_set_style(label, style);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);

	eventbox = gtk_event_box_new();

	gtk_signal_connect(GTK_OBJECT(eventbox), "button_press_event", GTK_SIGNAL_FUNC(ShowMenu), NULL);

	//MU style = gtk_style_copy(gtk_widget_get_style(eventbox));

	gtk_container_add(GTK_CONTAINER(eventbox), hbox);
	gtk_container_add(GTK_CONTAINER(window), eventbox);


    flashTimeout = 0;
    status = NORMAL;
    interval *= 1000;

    gtk_widget_show(GTK_WIDGET(window));

    gtk_timeout_add(interval, Update, NULL);

//MU	gtk_widget_show_all(window);


	Update(NULL);

	gtk_main();

	return 0;
}

/****************************************************************************
 ****************************************************************************/

void AppQuit(GtkWidget *widget, gpointer gdata) {
	gtk_main_quit();
}

/****************************************************************************
 * Gets a new memory value, updates the necessary stuff
 ****************************************************************************/

gboolean Update(gpointer ptr) {

    FILE *filePipe;
    char freeMemOutput[40];
	char freeMemString[9];

	char freeMemStringFormatted[100];
	char freeMemTip[40];
	unsigned long long freeMemKBytes;

    filePipe = (FILE *)popen("calcfreespace.sh","r"); /* use calcfreespace.sh to get the amount of free memory, in KBytes */
    fgets(freeMemOutput, sizeof(freeMemOutput), filePipe);
    pclose(filePipe);

	freeMemKBytes = strtoull(freeMemOutput, NULL, 10);
	FormatKBytes(freeMemKBytes, freeMemString, sizeof(freeMemString));

	if(freeMemKBytes > LOW_MEM*1024) {
		snprintf(freeMemTip, sizeof(freeMemTip), "Free Memory: %s", freeMemString);
		MemNormal();
	} else if(freeMemKBytes > LOW_MEM_FLASH*1024) {
		snprintf(freeMemTip, sizeof(freeMemTip), "Free Memory (LOW): %s", freeMemString);
		MemLow();
	} else {
		snprintf(freeMemTip, sizeof(freeMemTip), "Free Memory (CRITICAL): %s", freeMemString);
		MemLowFlash();
	}

//MU
//	gtk_label_set_text(GTK_LABEL(label), freeMemString);

//snprintf(freeMemStringFormatted, sizeof(freeMemStringFormatted), "<small>%s</small>", freeMemString);

char *ptr2;
int te=strtod(freeMemString ,&ptr2);

//snprintf(freeMemStringFormatted, sizeof(freeMemStringFormatted), "<span font_desc='10'>%i       </span>", te);

snprintf(freeMemStringFormatted, sizeof(freeMemString), "<span font_desc='8'>%i</span>", te);

        gtk_label_set_markup (GTK_LABEL (label), freeMemString);
	gtk_tooltips_set_tip(tips, eventbox, freeMemTip, "");

	return TRUE;
}

/****************************************************************************
 ****************************************************************************/

void MemNormal(void) {
	if(status == LOW_FLASH) {
		gtk_timeout_remove(flashTimeout);
	}

	if(!IsCurrentBackground(&colorNormal)) {
		SetBackground(&colorNormal);
	}

	status = NORMAL;
}

/****************************************************************************
 ****************************************************************************/

void MemLow(void) {
	if(status == LOW_FLASH) {
		gtk_timeout_remove(flashTimeout);
	}

	if(!IsCurrentBackground(&colorLow)) {
		SetBackground(&colorLow);
	}

	status = LOW;
}

/****************************************************************************
 ****************************************************************************/

void MemLowFlash(void) {
	if(status != LOW_FLASH) {
		Flash(NULL);
		flashTimeout = gtk_timeout_add(FLASH_INTERVAL, Flash, NULL);
		status = LOW_FLASH;
	}
}

/****************************************************************************
 ****************************************************************************/

gboolean Flash(gpointer ptr) {
	if(!IsCurrentBackground(&colorLowFlash1)) {
		SetBackground(&colorLowFlash1);
	} else {
		SetBackground(&colorLowFlash2);
	}

	return TRUE;
}

 /****************************************************************************
 ****************************************************************************/

gboolean IsCurrentBackground(GdkColor *color) {
	return gdk_color_equal(&(style->bg[GTK_STATE_NORMAL]), color);
}

 /****************************************************************************
 * Sets eventbox's background and redraws it
 ****************************************************************************/

void SetBackground(GdkColor *color) {
	style->bg[GTK_STATE_NORMAL] = *color;
	gtk_widget_set_style(eventbox, style);

	/* redraw to change the color */
	gtk_widget_hide_all(eventbox);
	gtk_widget_unrealize(eventbox);
	gtk_widget_realize(eventbox);
	gtk_widget_show_all(eventbox);
}

 /****************************************************************************
 ****************************************************************************/
 
void ShowMenu(GtkWidget *widget, GdkEventButton *event, gpointer dummy) {
	static GtkWidget *menu = NULL;
	GtkWidget *item;

	if(event->type == GDK_BUTTON_PRESS && (event->button == 3 || event->button == 1)) {
		if(menu)
			gtk_widget_destroy(menu);

		menu = gtk_menu_new();

		item = gtk_menu_item_new_with_label("Help: What does this mean?");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(RunCommand), "dillo /usr/share/doc/freememapplet.htm");
		gtk_widget_show(item);

		item = gtk_menu_item_new_with_label("View mounted partition sizes (partview)");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(RunCommand), "partview");
		gtk_widget_show(item);

		item = gtk_menu_item_new_with_label("Manage running processes (KP)");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(RunCommand), "kp");
		gtk_widget_show(item);

		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
	}
}

 /****************************************************************************
  * Runs the given command
 ****************************************************************************/

void RunCommand(GtkWidget *widget, char *command) {
	switch((fork())) {
		case -1:
			g_print("%s: fork failed, `%s` is not executed\n", APPNAME, command);
			return;
		case 0:
			execl("/bin/sh", "sh", "-c", command, NULL);
	}
}

 /****************************************************************************
 ****************************************************************************/

void ExposeUpdate(GtkWidget *widget, GdkEvent *event, gpointer dummy) {
	Update(NULL);
}

 /****************************************************************************
 ****************************************************************************/

void FormatKBytes(unsigned long long kbytes, char *buffer, int size) {
	if(kbytes > 0.0) { /* KB range  */
		if(kbytes >= 1024.0) { /* MB range  */
			if(kbytes >= 1048576.0) { /* GB range  */
				if(kbytes >= 1073741824.0) { /* TB range */
					if(kbytes >= 1099511627776.0) { /* PB range */
						if(kbytes >= 1125899906842624.0) { /* EB range */
							if(kbytes >= 1152921504606846976.0) { /* ZB range */
								if(kbytes >= 1180591620717411303424.0) { /* YB range */
									g_snprintf(buffer, size, 
									"%.1fY",
									(float)(kbytes/1180591620717411303424.0));
								} else  /* ZB range */
									g_snprintf(buffer, size, 
									"%.1fZ",
									(float)(kbytes/1152921504606846976.0));
							} else /* EB range */
								g_snprintf(buffer, size, 
								"%.1fE",
								(float)(kbytes/1125899906842624.0));
						} else /* PB range */
							g_snprintf(buffer, size, 
							"%.1fP",
							(float)(kbytes/1099511627776.0));
					} else /* TB range */
						g_snprintf(buffer, size, 
						"%.1fT",
						(float)(kbytes/1073741824.0));
				} else /* GB range */
					g_snprintf(buffer, size, 
					"%.1fG",
					(float)(kbytes/1048576.0));
			} else /* MB range */
				g_snprintf(buffer, size, 
				"%.1fM",
				(float)(kbytes/1024.0));
		} else /*  KB range */
			g_snprintf(buffer, size, 
			"%.1fK",
			(float)(kbytes));
	} else // none
		g_snprintf(buffer, size, 
		"None");
	return;
}

